//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function L = Link (varargin)
    // Robot manipulator Link class
    //
    // Calling Sequence   
    //     L = Link()
    //    
    //     L = Link(L1)
    //    
    //     L = Link(OPTIONS)
    //
    //	     //Old syntax:
    //     L = Link(DH,OPTIONS)
    //     DH = [THETA D A ALPHA]
    //
    //	Parameters
    //	OLD SINTAX : kinematic
    //	theta   :  joint angle
    //  d       :  link offset
    //	a       :  link length
    //  alpha   :  link twist
    //	sigma   :  0 if revolute, 1 if prismatic
    //	mdh     :  0 if standard D&H, else 1
    //
    //     OPTIONS		: Key/value pairs
    //     'theta',TH  : joint angle, if not specified joint is revolute
    //     'd',D       : joint extension, if not specified joint is prismatic
    //     'a',A       : joint offset (default 0)
    //     'alpha',A   : joint twist (default 0)
    //     'standard'  : defined using standard D&H parameters (default).
    //     'modified'  : defined using modified D&H parameters.
    //     'offset',O  : joint variable offset (default 0)
    //     'qlim',L    : joint limit (default [])
    //     'I',I       : link inertia matrix (3x1, 6x1 or 3x3)
    //     'r',R       : link centre of gravity (3x1)
    //     'm',M       : link mass (1x1)
    //     'G',G       : motor gear ratio (default 0)
    //     'B',B       : joint friction, motor referenced (default 0)
    //     'Jm',J      : motor inertia, motor referenced (default 0)
    //     'Tc',T      : Coulomb friction, motor referenced (1x1 or 2x1), (default [0 0])
    //     'revolute'  : for a revolute joint (default)
    //     'prismatic' : for a prismatic joint 'p'
    //     'standard'  : for standard D&H parameters (default).
    //     'modified'  : for modified D&H parameters.
    //
    //    Description
    // 	A Link object holds all information related to a robot link such as
    // 	kinematics parameters, rigid-body inertial parameters, motor and
    //	transmission parameters.
    //
    //	This is class constructor function which has several call signatures.
    //
    //	L = Link() is a Link object with default parameters.
    //
    //  L = Link(L1) is a Link object that is a deep copy of the link
    //  object L1.
    //
    //  L = Link(OPTIONS) is a link object with the kinematic and dynamic
    //  parameters specified by the key/value pairs.
    //
    //	<itemizedlist>
    //    <listitem><para>It is an error to specify 'theta' and 'd' </para></listitem>
    //    <listitem><para>The link inertia matrix (3x3) is symmetric and can be specified by giving
    //       a 3x3 matrix, the diagonal elements [Ixx Iyy Izz], or the moments and products
    //       of inertia [Ixx Iyy Izz Ixy Iyz Ixz].</para></listitem>
    //    <listitem><para>All friction quantities are referenced to the motor not the load.</para></listitem>
    //    <listitem><para>Gear ratio is used only to convert motor referenced quantities such as
    //       friction and interia to the link frame.</para></listitem>
    //	</itemizedlist>
    //
    //  Old syntax:
    //
    //   L = Link(DH, OPTIONS) is a link object using the specified kinematic
    //   convention  and with parameters:
    //
    //  - DH = [THETA D A ALPHA SIGMA OFFSET] where OFFSET is a constant displacement
    //   between the user joint angle vector and the true kinematic solution.
    //
    // - DH = [THETA D A ALPHA SIGMA] where SIGMA=0 for a revolute and 1 for a
    //   prismatic joint, OFFSET is zero.
    //
    // - DH = [THETA D A ALPHA], joint is assumed revolute and OFFSET is zero.
    //
    //             Options:
    //            
    //             'standard'   for standard D&H parameters (default).
    //
    //             'modified'   for modified D&H parameters.
    //
    //             'revolute'   for a revolute joint, can be abbreviated to 'r' (default)
    //
    //             'prismatic'  for a prismatic joint, can be abbreviated to 'p'
    //           
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    //    <listitem><para>Link object is a reference object, a subclass of Handle object.</para></listitem>
    //    <listitem><para>Link objects can be used in vectors and arrays.</para></listitem>
    //    <listitem><para>The parameter D is unused in a revolute joint, it is simply a placeholder
    //       in the vector and the value given is ignored.</para></listitem>
    //    <listitem><para>The parameter THETA is unused in a prismatic joint, it is simply a placeholder
    //       in the vector and the value given is ignored.</para></listitem>
    //    <listitem><para>The joint offset is a constant added to the joint angle variable before
    //       forward kinematics and subtracted after inverse kinematics.  It is useful
    //       if you  want the robot to adopt a 'sensible' pose for zero joint angle
    //       configuration.</para></listitem>
    //     <listitem><para>The link dynamic (inertial and motor) parameters are all set to
    //       zero.  These must be set by explicitly assigning the object
    //       properties: m, r, I, Jm, B, Tc, G.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //    
    //  Examples
    //     //A standard Denavit-Hartenberg link
    //            L3 = Link('d', 0.15005, 'a', 0.0203, 'alpha', -%pi/2);
    //     //since 'theta' is not specified the joint is assumed to be revolute, and
    //     //since the kinematic convention is not specified it is assumed 'standard'.
    //    
    //     //Using the old syntax
    //            L3 = Link( 0, 0.15005, 0.0203, -%pi/2, 0, 'standard');
    //     //the flag 'standard' is not strictly necessary but adds clarity.
    //    
    //     //For a modified Denavit-Hartenberg link
    //            L3 = Link( 0, 0.15005, 0.0203, -%pi/2, 0, 'modified');
    //
    // Bibliography
    // Robotics, Vision & Control, Chap 7, P. Corke, Springer 2011.    
    //
    // See also 
    //	Link
    //	SerialLink.
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci;

    //
    // This file is part of The Robotics Toolbox for Matlab (RTB).

    [%nargout, %nargin] = argn(0);

    //
    //     L = Link() is a Link object with default parameters.
    //
    
    a = 0;
    alpha = 0;
    d = 0;
    theta = 0;
    sigma = 0;
    mdh = 0;
    offset = 0;
    m = 0;
    r = [0 0 0];
    I = zeros(3,3);
    Jm = 0;
    G = 0;
    B = 0;
    Tc = [0 0];
    qlim = [];
    L = mlist(["Link","a","alpha","d","theta","sigma","mdh","offset","m","r","I","Jm","G","B","Tc","qlim"], a, alpha, d, theta, sigma, mdh, offset, m, r, I, Jm, G, B, Tc, qlim);

    if %nargin == 0  then             
        return;
    end

    //
    //     L = Link(L1) is a Link object that is a deep copy of the link
    //     object L1.
    //
    if (%nargin == 1 & typeof(varargin(1))=="Link")   then             
        // clone the passed Link object
        L = varargin(1);
        //        disp("Link cloned");
        return;
    end

    if (%nargin > 1 & typeof(varargin(1))=="Link")   then             
        // additional parameters must be added to an existing link
        L = varargin(1);
        i = 2;
    else
        i=1
    end

    //
    //     L = Link(OPTIONS) is a link object with the kinematic and dynamic
    //     parameters specified by the key/value pairs.
    //
    if typeof(varargin(i))=="string" then
        //     Key/value pairs
        while i<=length(varargin),
            if ~strcmp(string(varargin(i)),'r') then
                L.r = set_r(varargin(i+1));
            elseif  ~strcmp(string(varargin(i)),'a') then
                L.a = varargin(i+1);
            elseif ~strcmp(string(varargin(i)),'alpha')
                L.alpha = varargin(i+1);
            elseif ~strcmp(string(varargin(i)),'d')
                L.d = varargin(i+1);
            elseif ~strcmp(string(varargin(i)),'theta')
                L.sigma = 1;
                L.theta = varargin(i+1);	
            elseif ~strcmp(string(varargin(i)),'sigma')
                L.sigma = varargin(i+1);
            elseif (~strcmp(string(varargin(i)),'revolute') | ~strcmp(string(varargin(i)),'r'))
                L.sigma = 0;
                i=i-1;
            elseif (~strcmp(string(varargin(i)),'prismatic') | ~strcmp(string(varargin(i)),'p'))
                L.sigma = 1;
                i=i-1;                
            elseif ~strcmp(string(varargin(i)),'modified')
                L.mdh = 1;
                i=i-1;
            elseif ~strcmp(string(varargin(i)),'standard')
                L.mdh = 0;
                i=i-1;
            elseif ~strcmp(string(varargin(i)),'offset')
                L.offset = varargin(i+1);	
            elseif ~strcmp(string(varargin(i)),'I')
                L.I = set_I(varargin(i+1));
            elseif ~strcmp(string(varargin(i)),'m')
                L.m = set_m(varargin(i+1));		
            elseif ~strcmp(string(varargin(i)),'Jm')
                L.Jm = varargin(i+1);		
            elseif ~strcmp(string(varargin(i)),'G')
                L.G = varargin(i+1);		
            elseif (~strcmp(string(varargin(i)),'B') | ~strcmp(string(varargin(i)),'Bm'))
                L.B = varargin(i+1);		
            elseif ~strcmp(string(varargin(i)),'Tc')
                L.Tc = set_Tc(varargin(i+1));		
            elseif ~strcmp(string(varargin(i)),'qlim')
                L.qlim = varargin(i+1);	
            else
                error(msprintf(gettext('Link: Unkwnon argument''%s ''\n'),string(varargin(i))));
            end
            i=i+2;
        end           
        if (L.theta<>0 & L.d<>0) then
            error(msprintf(gettext('%s: specify only one of ''d'' or ''theta''\n'),'Link'));
        end
    elseif (length(varargin(1))>1 & typeof(varargin(1)) == "constant")
        //         This is the old call format, where all parameters are
        //         given by a vector containing kinematic-only.
        //        
        //        eg. L3 = Link([ 0, 0.15005, 0.0203, -pi/2, 0], 'standard');

        if length(varargin(1))<4 then
            error('Link: must provide params (theta d a alpha)');
        end
        L.theta = varargin(1)(1);
        L.d = varargin(1)(2);
        L.a = varargin(1)(3);
        L.alpha = varargin(1)(4);
        L.sigma = 0;
        L.offset = 0;
        L.mdh = 0; 
        if length(varargin(1)) >= 5
            L.sigma = varargin(1)(5);
        end
        if length(varargin(1)) == 6
            L.offset = varargin(1)(6);
        end
        if length(varargin) == 2 then
            if (~strcmp(varargin(2),'mod') | ~strcmp(varargin(2),'modified') | ~strcmp(varargin(2),'Modified') | ~strcmp(varargin(2),'Mod')) then
                L.mdh=1;
            else
                L.mdh=0;
            end
        end
        if length(varargin) > 2 then
            warning("LINK: old call format accept only one addictional arguments: DH type")
        end


    elseif (length(varargin(1))==1 & typeof(varargin(1)) == "constant")
        if  %nargin >= 4 then
            L.a = varargin(3);
            L.alpha = varargin(4);
            L.d = varargin(2);
            L.theta = varargin(1);
            L.sigma = 0;
            L.mdh = 0;
        end
        if %nargin == 5 & typeof(varargin(5))<>"string" then
            L.sigma = varargin(5);
        elseif %nargin == 6 then
            if (~strcmp(varargin(6),'mod') | ~strcmp(varargin(6),'modified') | ~strcmp(varargin(6),'Modified') | ~strcmp(varargin(6),'Mod')) then
                L.mdh=1;
            else
                L.mdh=0;
            end
        end
    else
        error('Link: bad arguments');
    end
endfunction
