//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function q = Quaternion(a1, a2)
    // Constructor for quaternion objects
    //
    // Calling Sequence
    // Q = Quaternion()
    // Q = Quaternion(Q1)
    // Q = Quaternion([S V1 V2 V3])
    // Q = Quaternion(S)
    // Q = Quaternion(TH, V)
    // Q = Quaternion(R)
    // Q = Quaternion(T)
    //
    // Description
    // Q = Quaternion() is the identitity quaternion 1(0,0,0) representing a null rotation.
    //
    // Q = Quaternion(Q1) is a copy of the quaternion Q1
    //
    // Q = Quaternion([S V1 V2 V3]) is a quaternion formed by specifying directly its 4 elements
    //
    // Q = Quaternion(S) is a quaternion formed from the scalar S and zero vector part: S(0,0,0)
    //
    // Q = Quaternion(V) is a pure quaternion with the specified vector part: 0(V)
    //
    // Q = Quaternion(TH, V) is a unit-quaternion corresponding to rotation of TH about the 
    // vector V.
    //
    // Q = Quaternion(R) is a unit-quaternion corresponding to the orthonormal rotation 
    // matrix R.  If R (3x3xN) is a sequence then Q (Nx1) is a vector of Quaternions 
    // corresponding to the elements of R.
    //
    // Q = Quaternion(T) is a unit-quaternion equivalent to the rotational
    // part of the homogeneous transform T. If T (4x4xN) is a sequence then Q (Nx1) is a 
    // vector of Quaternions corresponding to the elements of T.
    //
    // A quaternion is a compact method of representing a 3D rotation that has
    // computational advantages including speed and numerical robustness.
    // A quaternion has 2 parts, a scalar s, and a vector v and is typically 
    // written: q = s (vx, vy, vz).  
    // 
    // A unit-quaternion is one for which s^2+vx^2+vy^2+vz^2 = 1.  It can be 
    // considered as a rotation by an angle theta about a unit-vector V in space where 
    //    
    //        <literal> q = cos (theta/2) ( v sin(theta/2))  </literal>
    //
    // Q = Quaternion(X) is a unit-quaternion equivalent to X which can be any
    // of:
    //   - orthonormal rotation matrix.
    //
    //   - homogeneous transformation matrix (rotation part only).
    //
    //   - rotation angle and vector
    //
    // <refsection>
    // <title>Methods</title>
    // <variablelist>
    //   <varlistentry><term>inv     :</term>
    // 	<listitem><para>inverse of quaterion</para></listitem></varlistentry>
    //   <varlistentry><term>norm    :</term>
    // 	<listitem><para>norm of quaternion</para></listitem></varlistentry>
    //   <varlistentry><term>unit    :</term>
    // 	<listitem><para>unitized quaternion</para></listitem></varlistentry>
    //   <varlistentry><term>interp    :</term>
    // 	<listitem><para>interpolation (slerp) between q and q2, 0<=s<=1</para></listitem></varlistentry>
    //   <varlistentry><term>scale    :</term>
    // 	<listitem><para>interpolation (slerp) between identity and q, 0<=s<=1</para></listitem></varlistentry>
    //   <varlistentry><term>dot    :</term>
    // 	<listitem><para>derivative of quaternion with angular velocity w</para></listitem></varlistentry>
    //   <varlistentry><term>R    :</term>
    // 	<listitem><para>equivalent 3x3 rotation matrix</para></listitem></varlistentry>
    //   <varlistentry><term>T    :</term>
    // 	<listitem><para>equivalent 4x4 homogeneous transform matrix</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Arithmetic operators</title>
    // <variablelist>
    //   <varlistentry><term> q1==q2     :</term>
    // 	<listitem><para>test for quaternion equality</para></listitem></varlistentry>
    //   <varlistentry><term>q1~=q2    :</term>
    // 	<listitem><para>test for quaternion inequality</para></listitem></varlistentry>
    //   <varlistentry><term>q+q2    :</term>
    // 	<listitem><para>elementwise sum of quaternions</para></listitem></varlistentry>
    //   <varlistentry><term>q-q2    :</term>
    // 	<listitem><para>elementwise difference of quaternions</para></listitem></varlistentry>
    //   <varlistentry><term>q*q2    :</term>
    // 	<listitem><para>quaternion product</para></listitem></varlistentry>
    //   <varlistentry><term>q*v    :</term>
    // 	<listitem><para>rotate vector by quaternion, v is 3x1</para></listitem></varlistentry>
    //   <varlistentry><term>s*q    :</term>
    // 	<listitem><para>elementwise multiplication of quaternion by scalar</para></listitem></varlistentry>
    //   <varlistentry><term> qdivq2    :</term>
    // 	<listitem><para>q*q2.inv </para></listitem></varlistentry>
    //   <varlistentry><term> qelevaton    :</term>
    // 	<listitem><para>q to power n (integer only)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Properties (read only)</title>
    // <variablelist>
    //   <varlistentry><term> q1==q2     :</term>
    // 	<listitem><para>real part</para></listitem></varlistentry>
    //   <varlistentry><term>q1~=q2    :</term>
    // 	<listitem><para>vector part</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>Quaternion objects can be used in vectors and arrays</para></listitem>
    // </itemizedlist>
    // </refsection>
    //    
    // Bibliography
    // [1] Animating rotation with quaternion curves,  K. Shoemake, in Proceedings of ACM SIGGRAPH, (San Fran cisco), pp. 245-254, 1985.
    // [2] On homogeneous transforms, quaternions, and computational efficiency, J. Funda, R. Taylor, and R. Paul, IEEE Transactions on Robotics and Automation, vol. 6, pp. 382-388, June 1990.
    // [3] Robotics, Vision & Control, P. Corke, Springer 2011.
    //
    // See also 
    // trinterp 
    // trplot
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci


    [%nargout,%nargin]=argn(0);
    s = []; 
    v = [];
    q =tlist(["Quat",s,v]);
    if %nargin == 0 then
        q(3) = [0,0,0];
        q(2) = 1;
    elseif typeof(a1) == 'Quat'
        //   Q = Quaternion(q)       from another quaternion
        q = a1;
    elseif %nargin == 1
        if isvec(a1, 4) then
            //   Q = Quaternion([s v1 v2 v3])    from 4 elements
            a1 = a1(:);
            q(2) = a1(1);
            q(3) = a1(2:4)';
        elseif (isrot(a1) | ishomog(a1))
            //   Q = Quaternion(R)       from a 3x3 or 4x4 matrix
            if size(a1,3)>1 then
                q = list();
                for i=1:size(a1,3)
                    q(i) = Quaternion(tr2q(a1(:,:,i)));
                end
            else
                q = Quaternion(tr2q(a1));
            end
        elseif length(a1) == 3
            //   Q = Quaternion(v)       from a vector
            q(2) = 0;
            q(3) = a1(:)';
        elseif length(a1) == 1
            //   Q = Quaternion(s)       from a scalar
            q(2) = a1(1);
            q(3) = [0 0 0];
        else
            error('unknown dimension of input');
        end
    elseif %nargin == 2
        if (sum(length(a1))==1 & (size(a2,1)==1 | size(a2,2)==1)) then
            //   Q = Quaternion(theta, v)    from vector plus angle
            q(2) = cos(a1/2);
            q(3) = sin(a1/2)*qunit(a2(:)');
        else
            error ('bad argument to quaternion constructor');
        end

    end
endfunction
