//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [m, mx] = asada(robot, q, opt)
// Manipulability measure
//
// Calling Sequence
// M = maniplty(R, Q, OPTIONS, 'asada')
// [M,CI] = maniplty(R, Q, OPTIONS,' asada')
//
// Parameters
// R			: robot object
// Q			: matrix or vector, joint configuration.
// 'T'          : manipulability for transational motion only (default)
// 'R'          : manipulability for rotational motion only
// 'all'        : manipulability for all motions
// 'dof',D      : D is a vector (1x6) with non-zero elements if the
//               corresponding DOF is to be included for manipulability
// 'yoshikawa'  : use Yoshikawa algorithm (default)
// 'asada'      : use Asada algorithm
//
// Description
// Asada's manipulability measure is based on the shape of the acceleration
// ellipsoid which in turn is a function of the Cartesian inertia matrix and
// the dynamic parameters.  The scalar measure computed here is the ratio of 
// the smallest/largest ellipsoid axis.  Ideally the ellipsoid would be 
// spherical, giving a ratio of 1, but in practice will be less than 1.
//
// If Q is a matrix (MxN) then M (Mx1) is a vector of  manipulability 
// indices for each pose specified by a row of Q.
// [M,CI] = maniplty(R,Q, OPTIONS, 'asada') as above, but for the case of the Asada
// measure returns the Cartesian inertia matrix CI.
// 
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>By default the measure includes rotational and translational dexterity, but
// this involves adding different units.  It can be more useful to look at the
// translational and rotational manipulability separately.</para></listitem>
// </itemizedlist>
// </refsection>
//
// Bibliography
// [1] Analysis and control of robot manipulators with redundancy, T. Yoshikawa, Robotics Research: The First International Symposium (M. Brady and R. Paul, eds.), pp. 735-747, The MIT press, 1984.
// [2] A geometrical representation of manipulator dynamics and its application to arm design, H. Asada, Journal of Dynamic Systems, Measurement, and Control, vol. 105, p. 131, 1983.
//
// See also
// SerialLink
// maniplty
// yoshi
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    [%nargout, %nargin] = argn(0);
    J = jacob0(robot,q);

    if rank(J) < 6
        warning('robot is in degenerate configuration')
        m = 0;
        return;
    end

    Ji = pinv(J);
    M = inertia(robot,q);
    Mx = Ji' * M * Ji;
    d = find(opt.dof);
    Mx = Mx(d,d);
    e = spec(Mx);
    m = min(real(e)) / max(real(e));

    if %nargout > 1
        mx = Mx;
    end
endfunction

