//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function R = eul2r(alpha, varargin)
    // Euler angles to rotation matrix
    //
    // Calling Sequence
    // R = eul2r(ANG, OPTIONS)
    // R = eul2r(ALPHA, BETA, GAMMA, OPTIONS)
    //
    // Description
    // R = EUL2R(ANG, OPTIONS) is an orthonormal rotation matrix equivalent to the 
    // specified alpha, beta, gamma Euler angles which correspond to rotations about the 
    // X, Y, Z axes respectively. If ANG has multiple rows they are assumed to 
    // represent a trajectory and R is a three dimensional matrix, where the last index  
    // corresponds to the rows of RPY.
    //
    // R = EUL2R(ALPHA, BETA, GAMMA, OPTIONS) as above but the alpha, beta, gamma Euler angles 
    // are passed as separate arguments. If ALPHA, BETA, GAMMA are column vectors 
    // they are assumed to represent a trajectory and R is a three dimensional matrix,
    // where the last index corresponds to the rows of ALPHA, BETA, GAMMA.
    //
    // <refsection>
    //   <title>OPTIONS</title>
    //   <variablelist>
    //   <varlistentry><term>"deg" :</term>
    //         <listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    //  </variablelist>
    // </refsection>
    //
    // See also 
    // tr2rpy
    // zyz2tr
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco
    
    funcprot(0);
    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if ~strcmp(string(varargin(i)),'deg')
                opt.deg = ~opt.deg;
            elseif typeof(varargin(i))=="string"
                warning(msprintf(gettext('%s: Unkwnon argument ''%s''\n'),'eul2r',varargin(i)));
            end
        end
    end

    // unpack the arguments
    if numcols(alpha) == 3
        beta = alpha(:,2);
        gamma = alpha(:,3);
        alpha = alpha(:,1);
    elseif %nargin >= 3
        beta = varargin(1);
        gamma = varargin(2);
    else
        error('bad arguments')
    end

    // optionally convert from degrees
    if opt.deg
        d2r = %pi/180.0;
        alpha = alpha * d2r;
        beta = beta * d2r;
        gamma = gamma * d2r;
    end

    if numrows(alpha) == 1
        R = rotx(alpha) * roty(beta) * rotz(gamma);
    else
        R = zeros(3,3,numrows(alpha));
        for i=1:numrows(alpha)
            R(:,:,i) = rotx(alpha(i)) * roty(beta(i)) * rotz(gamma(i));
        end
    end
 
 endfunction
