//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function  tau = friction(robot, qd)
// Friction force
//
// Calling Sequence
// TAU = friction(ROBOT,QD)
// TAU = friction(L,QD)
//    
// Parameters
// L    : Link object
// ROBOT : SerialLink object
// QD   : vector of size (1xN) where N is the number of joints of the robot.
//     
// Description
// <literal>TAU = friction(L,QD)</literal> is the joint friction force/torque for link velocity QD.
//
// <literal>TAU = friction(ROBOT,QD)</literal> is the vector of joint friction forces/torques for the 
// robot moving with joint velocities QD.  
//
// The friction model includes:
// - viscous friction which is linear with velocity;
//
// - Coulomb friction which is proportional to sign(QD).
//
// <refsection>
// <title>Notes</title>
//  <itemizedlist>
//  <listitem><para> Friction values are referred to the motor, not the load.</para></listitem>
//  <listitem><para> Viscous friction is scale   d up by G^2.</para></listitem>
//  <listitem><para> Coulomb friction is scaled up by G.</para></listitem>
//  <listitem><para> The sign of the gear ratio is used to determine the appropriate
//  Coulomb friction value in the non-symmetric case.</para></listitem>
//  </itemizedlist> 
// </refsection>  
// 
// Bibliography
// [1] Robotics, Vision & Control, Chap 7, P. Corke, Springer 2011.   
//
// See also
//  Link 
//	fdyn
//  nofriction
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    if  typeof(robot)== "Robot" then
        L = robot.links;

        tau = zeros(1,robot.n);
        //    if robot.issym
        //        tau = sym(tau);
        //    end


        for j=1:robot.n
            tau(j) = friction(L(j),qd(j));
        end
    elseif typeof(robot)== "Link" then
        l = robot;
        tau = l.B * abs(l.G) * qd;
        //if issym(l) then
        //tau = tau + l.Tc;
        if qd > 0 then
            tau = tau + l.Tc(1);
        elseif qd < 0 then
            tau = tau + l.Tc(2);
        end;
        tau = -abs((l.G))*tau;  // friction opposes motion
    else
        error(mprintf("Unknown type.\nUsage: friction(link,qd) or friction(robot,qd)"));
    end

endfunction
