//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function theta = ikine3(robot, T, varargin)
// Inverse kinematics for 3-axis robot with no wrist
//
// Calling Sequence
// Q = ikine3(R, T)
// Q = ikine3(R, T, CONFIG)
//
// Parameters
// CONFIG : valid character
// 'l'   : arm to the left (default)
// 'r'   : arm to the right
// 'u'   : elbow up (default)
// 'd'   : elbow down
//
// Description
// Q = ikine3(R, T) is the joint coordinates corresponding to the robot
// end-effector pose T represented by the homogenenous transform.  This
// is a analytic solution for a 3-axis robot (such as the first three joints
// of a robot like the Puma 560).
//
// Q = ikine3(R, T, CONFIG) as above but specifies the configuration of the arm in
// the form of a string containing one or more of the configuration codes:
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>The same as IKINE6S without the wrist.</para></listitem>
// <listitem><para>The inverse kinematic solution is generally not unique, and 
// depends on the configuration string.</para></listitem>
// <listitem><para>Joint offsets, if defined, are added to the inverse kinematics to
//  generate Q.</para></listitem>
// <listitem><para>If the robot has only three joints the tool position must be 
// specified through the tool frame homogeneous transform, otherwise it is posed
// equal to the origin of frame 4.</para></listitem>
// </itemizedlist>
// </refsection>
//
// Bibliography
// Inverse kinematics for a PUMA 560 based on the equations by Paul and Zhang, 
// The International Journal of Robotics Research, Vol. 5, No. 2, Summer 1986, p. 32-44
//
// See also 
// ikine
// ikin6s
//
// Authors
// Robert Biro with Gary Von McMurray, GTRI/ATRP/IIMB, Georgia Institute of Technology
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci
// Copyright (C) 2015 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    if length(robot.config) > 3 then
        config = strsplit(robot.config,3);
        config = config(1);
    else
        config = robot.config
    end

    if config ~= 'RRR' then
        error('Solution Only Applicable For 3DoF All-Revolute Manipulator');
    end

    if robot.mdh ~= 0
        error('Solution only applicable for standard DH conventions');
    end

    if ndims(T) == 3
        theta = zeros(size(T,3),robot.n);
        for k=1:size(T,3)
            if %nargin == 2 then
                theta(k,:) = ikine3(robot, T(:,:,k))';
            else
                theta(k,:) = ikine3(robot, T(:,:,k), varargin(:))';
            end
        end
        return;
    end

    L = robot.links;
    a2 = L(2).a;
    a3 = L(3).a;

    d3 = L(3).d;
    if length(robot.config) > 3 then
        d4 = L(4).d;
    else
        d4 = robot.tool(3,4);
    end

    if ~ishomog(T)
        error('T is not a homog xform');
    end

    // undo base transformation
    T = robot.base \ T;

    // The following parameters are extracted from the Homogeneous 
    // Transformation as defined in equation 1, p. 34

    Px = T(1,4);
    Py = T(2,4);
    Pz = T(3,4);

    // The configuration parameter determines what n1,n2 values are used
    // and how many solutions are determined which have values of -1 or +1.

    if %nargin < 3
        configuration = '';
    else
        configuration = convstr(varargin(1), 'l');
    end

    // default configuration

    n1 = -1;    // L
    n2 = -1;    // U

    if ~isempty(strindex(configuration, 'l'))
        n1 = -1;
    end
    if ~isempty(strindex(configuration, 'r'))
        n1 = 1;
    end
    if ~isempty(strindex(configuration, 'u'))
        if n1 == 1
            n2 = 1;
        else
            n2 = -1;
        end
    end
    if ~isempty(strindex(configuration, 'd'))
        if n1 == 1
            n2 = -1;
        else
            n2 = 1;
        end
    end

    //
    // Solve for theta(1)
    // 
    // r is defined in equation 38, p. 39.
    // theta(1) uses equations 40 and 41, p.39, 
    // based on the configuration parameter n1
    //

    r=sqrt(Px^2 + Py^2);
    if (n1 == 1)
        theta(1)= atan(Py,Px) + asin(d3/r);
    else
        theta(1)= atan(Py,Px) + %pi - asin(d3/r);
    end

    //
    // Solve for theta(2)
    //
    // V114 is defined in equation 43, p.39.
    // r is defined in equation 47, p.39.
    // Psi is defined in equation 49, p.40.
    // theta(2) uses equations 50 and 51, p.40, based on the configuration 
    // parameter n2
    //

    V114= Px*cos(theta(1)) + Py*sin(theta(1));
    r=sqrt(V114^2 + Pz^2);
    Psi = acos((a2^2-d4^2-a3^2+V114^2+Pz^2)/(2.0*a2*r));
    if ~isreal(Psi)
        warning('RTS:ikine3:notreachable:   point not reachable');
        theta = [%nan %nan %nan %nan %nan %nan];
        return
    end
    theta(2) = atan(Pz,V114) + n2*Psi;

    //
    // Solve for theta(3)
    //
    // theta(3) uses equation 57, p. 40.
    //

    num = cos(theta(2))*V114+sin(theta(2))*Pz-a2;
    den = cos(theta(2))*Pz - sin(theta(2))*V114;
    theta(3) = atan(a3,d4) - atan(num, den);

    for i=4:robot.n
        theta(i) = 0;
    end

    // remove the link offset angles
    for i=1:robot.n   //#ok<*AGROW>
        theta(i) = theta(i) - L(i).offset;
    end

endfunction
