//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function mdl_puma560akb()
    // Create model of Puma 560 manipulator
    //
    // Calling Sequence
    // mdl_puma560akb
    //
    // Description
    // Script creates the workspace variable p560m which describes the 
    // kinematic and dynamic characterstics of a Unimation Puma 560 manipulator 
    // modified DH conventions.
    //
    // Also defines the workspace vectors:
    //    
    //   qz         zero joint angle configuration
    //    
    //   qready         vertical 'READY' configuration
    //    
    //   qstretch   arm is stretched out in the X direction
    //
    // Bibliography
    // [1] "The Explicit Dynamic Model and Inertial Parameters of the Puma 560 Arm" - Armstrong, Khatib and Burdick - 1986
    //    
    // See also 
    // SerialLink
    // mdl_puma560 
    // mdl_stanford
    // mdl_twolink
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci


    clear L
    L = list();
    //            theta    d        a    alpha
    L(1) = Link([  0      0        0       0       0], 'modified');
    L(2) = Link([  0      0.2435   0      -%pi/2    0], 'modified');
    L(3) = Link([  0     -0.0934   0.4318  0       0], 'modified');
    L(4) = Link([  0      0.4331  -0.0203  %pi/2    0], 'modified');
    L(5) = Link([  0      0        0      -%pi/2    0], 'modified');
    L(6) = Link([  0      0        0       %pi/2    0], 'modified');


    L(1).m = 0;
    L(2).m = 17.4;
    L(3).m = 4.8;
    L(4).m = 0.82;
    L(5).m = 0.34;
    L(6).m = .09;

    //         rx      ry      rz
    L(1).r = set_r([0   0   0 ]);
    L(2).r = set_r([0.068   0.006   -0.016]);
    L(3).r = set_r([0   -0.070  0.014 ]);
    L(4).r = set_r([0   0   -0.019]);
    L(5).r = set_r([0   0   0 ]);
    L(6).r = set_r([0   0   .032  ]);

    //        Ixx     Iyy      Izz    Ixy     Iyz     Ixz
    L(1).I = set_I([0   0   0.35    0   0   0]);
    L(2).I = set_I([.13   .524    .539    0     0   0]);
    L(3).I = set_I([.066    .0125   .066    0   0   0]);
    L(4).I = set_I([1.8e-3  1.8e-3  1.3e-3  0   0   0]);
    L(5).I = set_I([.3e-3   .3e-3   .4e-3   0   0   0]);
    L(6).I = set_I([.15e-3  .15e-3  .04e-3  0   0   0]);

    L(1).Jm =  291e-6;
    L(2).Jm =  409e-6;
    L(3).Jm =  299e-6;
    L(4).Jm =  35e-6;
    L(5).Jm =  35e-6;
    L(6).Jm =  35e-6;

    L(1).G =  -62.6111;
    L(2).G =  107.815;
    L(3).G =  -53.7063;
    L(4).G =  76.0364;
    L(5).G =  71.923;
    L(6).G =  76.686;


    L(1).B = 1.48e-3;
    L(2).B = .817e-3;
    L(3).B = 1.38e-3;
    L(4).B = 71.2e-6;
    L(5).B = 82.6e-6;
    L(6).B = 36.7e-6;

    L(1).Tc = set_Tc([0.395, -0.435]);
    L(2).Tc = set_Tc([0.126, -0.071]);
    L(3).Tc = set_Tc([0.132, -0.105]);
    L(4).Tc = set_Tc([11.2e-3, -16.9e-3]);
    L(5).Tc = set_Tc([9.26e-3, -14.5e-3]);
    L(6).Tc = set_Tc([3.96e-3, -10.5e-3]);
    
    L(1).offset = 0;
    L(2).offset = 0;
    L(3).offset = 0;
    L(4).offset = 0;
    L(5).offset = 0;
    L(6).offset = 0;

    //
    // some useful poses
    //
    qz = [0 0 0 0 0 0]; // zero angles, L shaped pose
    qready = [0 -%pi/2 %pi/2 0 0 0]; // ready pose, arm up
    qstretch = [0 0 %pi/2 0 0 0]; // horizontal along x-axis

    p560m = SerialLink(L, 'name', 'Puma560-AKB', 'manufacturer', 'Unimation', 'comment', 'AK&B');
    [p560m,qz,qready,qstretch]=resume(p560m,qz,qready,qstretch)
endfunction
