//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [T] = rt2tr(R,t)
    // Convert rotation and translation to homogeneous transform
    // 
    // Calling Sequence
    // TR = rt2tr(R, t)
    //    
    // Description    
    // TR = RT2TR(R, t) is a homogeneous transformation matrix (MxM) formed from an 
    // orthonormal rotation matrix R (NxN) and a translation vector t (Nx1) where
    // M=N+1.
    // 
    // For a sequence R (NxNxK) and t (kxN) results in a transform sequence (NxNxk).
    // 
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para> Works for R in SO(2) or SO(3) </para></listitem>
    // <listitem><para> If R is 2x2 and t is 2x1, then TR is 3x3 </para></listitem>
    // <listitem><para> If R is 3x3 and t is 3x1, then TR is 4x4 </para></listitem>
    // <listitem><para> The validity of R is not checked </para></listitem>
    // </itemizedlist>
    // </refsection>
    // 
    // See also 
    // t2r
    // r2t 
    // tr2rt
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    if numcols(R)~=numrows(R) then
        error("R must be square");
    end;
    if numrows(R)~=numrows(t) then
        error("R and t must have the same number of rows");
    end;


    if size(R,3)<>numcols(t) then
        error("For sequence size(R,3) must equal size(t,2)");
    end;


    if size(R,3)>1 then
        Z = zeros(numcols(R),1);
        B = [Z',1];
        T = zeros(4,4,size(R,3));

        for i = 1:size(R,3)
            T(:,:,i) = [R(:,:,i),t(:,i);B];
        end;
    else
        T = [R,t;zeros(1,numcols(R)),1];
    end;

endfunction
