//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function ang = tr2eul(m, varargin)
    // Convert a homogeneous transform to alpha, beta, gamma Euler angles
    //
    // Calling Sequence
    // ANG = tr2eul(T, options)
    // ANG = tr2eul(R, options)
    //    
    // Description
    // ANG = TR2EUL(T, options) are the alpha, beta, gamma Euler angles expressed as a row 
    // vector corresponding to the rotation part of a homogeneous transform T.
    // The 3 angles ANG=[ALPHA, BETA, GAMMA] correspond to sequential rotations about 
    // the X, Y and Z mobile axes respectively.
    //
    // ANG = TR2EUL(R, options) are the alpha, beta, gamma Euler angles expressed as a row 
    // vector corresponding to the orthonormal rotation matrix R.
    //
    // If R or T represents a trajectory (has 3 dimensions), then each row of RPY
    // corresponds to a step of the trajectory.
    //
    // <refsection>
    // <title>Options</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>There is a singularity for the case where beta=pi/2 in which case 
    // alpha is arbitrarily set to zero and gamma is the sum (alpha+gamma).</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also  
    // eul2tr
    // rts_c_eul2tr
    // rts_c_tr2eul
    // rpy2tr
    // 
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    flag_c = %f

    out =list();
    for i = 1:(%nargin-1),
        if ~strcmp(string(varargin(i)),'c') then
            flag_c = %t;
        else
            out($+1) = varargin(i);
        end
        if ~strcmp(string(varargin(i)),'deg') then
            opt.deg = %t;
        end
    end

    if flag_c then
        try
            m = matrix(m,[4 4*size(m,3)]);
            ang = rts_c_tr2eul(m,out(:));
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_tr2eul not found.\n"),"tr2eul"));
            s = size(m);
            if length(s) > 2    then
                ang = zeros(s(3), 3);
                for i=1:s(3)
                    ang(i,:) = tr2eul(m(:,:,i), out(:));
                end
                return
            end
        end
    end

    s = size(m);
    if length(s) > 2    then
        ang = zeros(s(3), 3);
        for i=1:s(3)
            ang(i,:) = tr2eul(m(:,:,i), out(:));
        end
        return
    end

    ang = zeros(1,3);

    if abs(m(3,3)) < %eps & abs(m(2,3)) < %eps  then
        // singularity
        ang(1) = 0;  // alpha is zero
        ang(2) = atan(m(1,3), m(3,3));  // beta
        ang(3) = atan(m(2,1), m(2,2));  // gamma is sum of alpha+gamma
    else
        ang(1) = atan(-m(2,3), m(3,3));        // alpha
        // compute sin/cos of alpha angle
        sr = sin(ang(1));
        cr = cos(ang(1));
        ang(2) = atan(m(1,3), cr * m(3,3) - sr * m(2,3));  // beta
        ang(3) = atan(-m(1,2), m(1,1));        // gamma
    end

    if opt.deg  then
        ang = ang * 180/%pi;
    end
endfunction
