//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function rpy = tr2rpy(m, varargin)
    // Convert a homogeneous transform to roll-pitch-yaw angles
    //
    // Calling Sequence
    // RPY = tr2rpy(T, options)
    // RPY = tr2rpy(R, options)
    //    
    // Description
    // RPY = TR2RPY(T, options) are the roll-pitch-yaw angles expressed as a row 
    // vector corresponding to the rotation part of a homogeneous transform T.
    // The 3 angles RPY=[R,P,Y] correspond to sequential rotations about 
    // the X, Y and Z axes respectively.
    //
    // RPY = TR2RPY(R, options) are the roll-pitch-yaw angles expressed as a row 
    // vector corresponding to the orthonormal rotation matrix R.
    //
    // If R or T represents a trajectory (has 3 dimensions), then each row of RPY
    // corresponds to a step of the trajectory.
    //
    // <refsection>
    // <title>Options</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>There is a singularity for the case where P=pi/2 in which case 
    // R is arbitrarily set to zero and Y is the sum (R+Y).</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also  
    // rpy2tr
    // rts_c_rpy2tr
    // rts_c_tr2rpy
    // tr2eul.
    // 
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    flag_c = %f

    out =list();
    for i = 1:(%nargin-1),
        if ~strcmp(string(varargin(i)),'c') then
            flag_c = %t;
        else
            out($+1) = varargin(i);
        end
        if ~strcmp(string(varargin(i)),'deg') then
            opt.deg = %t;
        end
    end

    if flag_c then
        try
            m = matrix(m,[4 4*size(m,3)]);
            rpy = rts_c_tr2rpy(m,out(:));
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_tr2rpy not found.\n"),"tr2rpy"));
            s = size(m);
            if length(s) > 2    then
                rpy = zeros(s(3), 3);
                for i=1:s(3)
                    rpy(i,:) = tr2rpy(m(:,:,i), out(:));
                end
                return
            end
        end
    end
    
    s = size(m);
    if length(s) > 2    then
        rpy = zeros(s(3), 3);
        for i=1:s(3)
            rpy(i,:) = tr2rpy(m(:,:,i), varargin(:));
        end
        return
    end
    rpy = zeros(1,3);

    if abs(m(1,1)) < %eps & abs(m(2,1)) < %eps  then
        // singularity
        rpy(1) = 0;     // roll is zero
        rpy(2) = atan(-m(3,1), m(1,1));  // pitch
        rpy(3) = atan(-m(2,3), m(2,2));  // yaw is difference yaw-roll
    else
        rpy(1) = atan(m(2,1), m(1,1));
        sp = sin(rpy(1));
        cp = cos(rpy(1));
        rpy(2) = atan(-m(3,1), cp * m(1,1) + sp * m(2,1));
        rpy(3) = atan(sp * m(1,3) - cp * m(2,3), cp*m(2,2) - sp*m(1,2));
    end

    if opt.deg  then
        rpy = rpy * 180/%pi;
    end
endfunction
