//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = zyz2tr(phi, varargin)
// Euler angles to homogeneous transform
//
// Calling Sequence
// TR = ZYZ2TR(PHI, THETA, PSI, OPTIONS)
// TR = ZYZ2TR(EUL, OPTIONS)
//
// Description
// TR = ZYZ2TR(PHI, THETA, PSI, OPTIONS) is a homogeneous transformation
// equivalent to the specified Euler angles. These correspond to rotations 
// about the Z, Y, Z axes respectively. If PHI, THETA, PSI are column vectors 
// then they are assumed to represent a trajectory and R is a three dimensional 
// matrix, where the last index corresponds to rows of PHI, THETA, PSI.
//
// TR = ZYZ2TR(EUL, OPTIONS) as above but the Euler angles are taken from
// consecutive columns of the passed matrix EUL = [PHI THETA PSI].
//
// <refsection>
// <title>Options</title>
// <variablelist>
//   <varlistentry><term>'deg'     :</term>
// 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
//   <varlistentry><term>'c'       :</term>
// 	<listitem><para>Computes TR by means of a compiled c-function (faster)</para></listitem></varlistentry>
// </variablelist>
// </refsection>
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>Vectors PHI, THETA, PSI must be of the same length.</para></listitem>
// <listitem><para>The translational part is zero.</para></listitem>
// <listitem><para>TR can be evaluated more efficiently by means of c_zyz2tr.</para></listitem>
// </itemizedlist>
// </refsection>
//
// See also 
// zyz2r
// rpy2tr
// tr2zyz
// rts_c_zyz2tr
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci
// Copyright (C) 2014 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    flag = %f
    out =list();
    for i = 1:(%nargin-1),
        if strcmp(string(varargin(i)),'c') then
            out($+1) = varargin(i);
        else
            flag = %t
        end
    end
    
    if flag then
        try
            T = rts_c_zyz2tr(phi,out(:));
            T = matrix(T,[4 4 numrows(phi)]);
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_zyz2tr not found.\n"),"zyz2tr"));
            R = zyz2r(phi, out(:));
            T = r2t(R);
            return;
        end
    end

    R = zyz2r(phi, varargin(:));
    T = r2t(R);

endfunction
