/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include "api_scilab.h"
#include "Scierror.h"
#include "malloc.h"
#include "string.h"
#include "rpy2tr.h"
#include <localization.h>

/* ==================================================================== */
int rts_rpy2tr(char *fname, void* pvApiCtx)
{
    SciErr sciErr;

    double *newMatrixOfDouble = NULL;
    
    char string1[3] = "deg"; 
    char *string2;

    int i = 0;
    
    int m1 = 0, n1 = 0;
    int *piAddressVarOne = NULL;
    double *pdVarOne = NULL;
    int iType1 = 0;

    int m2 = 0, n2 = 0;
    int *piAddressVarTwo = NULL;
    double *pdVarTwo = NULL;
    int iType2 = 0;

    int m3 = 0, n3 = 0;
    int *piAddressVarThree = NULL;
    double *pdVarThree = NULL;
    int iType3 = 0;

    int m4 = 0, n4 = 0;
    int *piAddressVarFour = NULL;
    double *pdVarFour = NULL;
    int iType4 = 0;
    
    int m_out = 0, n_out = 0;
    double dOut = 0.0;

    /* check that we have only 3 input arguments */
    /* check that we have only 1 output argument */
    CheckInputArgument(pvApiCtx, 1, 4);
    CheckOutputArgument(pvApiCtx, 1, 1);

	switch (*getNbInputArgument(pvApiCtx))
    {
    case 1:
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A matrix is expected.\n"), fname, 1);
        	return 0;
    	}

    
    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( n1 != 3 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A three columns matrix is expected.\n"), fname, 1);
        	return 0;
    	}
    
		pdVarTwo = &pdVarOne[m1];
		pdVarThree = &pdVarOne[2*m1];
		break;
		
	case 2:
	
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A matrix expected.\n"), fname, 1);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarTwo, &iType2);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType2 != sci_strings )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A string expected.\n"), fname, 2);
        	return 0;
    	}

    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( n1 != 3 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A three columns matrix is expected.\n"), fname, 1);
        	return 0;
    	}
    
		pdVarTwo = &pdVarOne[m1];
		pdVarThree = &pdVarOne[2*m1];

    /* get string */
    	getAllocatedSingleString(pvApiCtx, piAddressVarTwo, &string2);

    /* check string */
    	if ( strncmp(string1,string2,3) )
    	{
        	Scierror(999, _("%s: Wrong input argument #%d: ""deg"" is expected.\n"), fname, 2);
        	return 0;
    	}

	    for (i = 0; i < m1 * n1; i++)
	    {
	        /* For each element of the matrix, multiply by pi/180.0 */
	        pdVarOne[i] = pdVarOne[i]*M_PI/180.0;
	    }    
		break;
		
	case 3:
    
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddressVarThree);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A scalar expected.\n"), fname, 1);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarTwo, &iType2);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType2 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A scalar expected.\n"), fname, 2);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarThree, &iType3);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType3 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A scalar expected.\n"), fname, 3);
        	return 0;
    	}
    
    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarTwo, &m2, &n2, &pdVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarThree, &m3, &n3, &pdVarThree);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( n1 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 1);
        	return 0;
    	}
    	if ( n2 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 2);
        	return 0;
    	}

    	if ( n3 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 3);
        	return 0;
    	}
    
    	if ( (m1 != m2) || ( m2 != m3) )
    	{
        	Scierror(999, _("%s: Wrong size for input argument. Column vectors must have the same size.\n"), fname);
        	return 0;
    	}
    	break;
    	
    case 4:
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddressVarThree);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 4, &piAddressVarFour);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: An array expected.\n"), fname, 1);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarTwo, &iType2);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType2 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: An array expected.\n"), fname, 2);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarThree, &iType3);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType3 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: An array expected.\n"), fname, 3);
        	return 0;
    	}
    
    	sciErr = getVarType(pvApiCtx, piAddressVarFour, &iType4);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType4 != sci_strings )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A string expected.\n"), fname, 4);
        	return 0;
    	}

    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarTwo, &m2, &n2, &pdVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarThree, &m3, &n3, &pdVarThree);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( n1 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 1);
        	return 0;
    	}
    	if ( n2 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 2);
        	return 0;
    	}

    	if ( n3 != 1 )
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A column vector is expected.\n"), fname, 3);
        	return 0;
    	}
    
    	if ( (m1 != m2) || ( m2 != m3) )
    	{
        	Scierror(999, _("%s: Wrong size for input argument. Column vectors must have the same size.\n"), fname);
        	return 0;
    	}
    	
    /* get string */
    	getAllocatedSingleString(pvApiCtx, piAddressVarFour, &string2);

    /* check string */
    	if ( strncmp(string1,string2,3) )
    	{
        	Scierror(999, _("%s: Wrong input argument #%d: ""deg"" is expected.\n"), fname, 4);
        	return 0;
    	}

	    for (i = 0; i < m1; i++)
	    {
	        /* For each element of the matrix, multiply by pi/180.0 */
	        pdVarOne[i] = pdVarOne[i]*M_PI/180.0;
	        pdVarTwo[i] = pdVarTwo[i]*M_PI/180.0;
	        pdVarThree[i] = pdVarThree[i]*M_PI/180.0;
	    }
	    
    	break;
    	
   	default :
		Scierror(999, _("%s: Wrong number of input arguments: please consult the function help.\n"), fname);
        return 0;
	}
	
    newMatrixOfDouble = (int*)malloc(sizeof(double) * 4 * (m1 * 4) );
    /* call c function rpy2tr */
    for (i = 0; i < m1; i++)
        rpy2tr(&pdVarOne[i], &pdVarTwo[i], &pdVarThree[i], &newMatrixOfDouble[16*i]);

    /* create result on stack */
    m_out = 4;
    n_out = m1*4;
    createMatrixOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + 1, m_out, n_out, newMatrixOfDouble);

    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;

    ReturnArguments(pvApiCtx);
    
    free(newMatrixOfDouble);

    return 0;
}
/* ==================================================================== */

