/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#ifndef MATH_HDR
#define MATH_HDR
/**
 * @defgroup math MATH
 */
/** @{ */
/**
 * @brief Enumeration for matrices
 */
enum math_enum_ {

    MATH_NOTRANS,  /**< no transpose */
    MATH_TRANS     /**< transpose */    

};

/**
 * @brief Data structure describing a vector with 3 elements
 */
typedef struct math_vect3_ {

    double data[3];    /**< vector data */

} vect3;

/**
 * @brief Set the X-axis component of an vect3 object
 * @param[out] vect modified vect3 object
 * @param[in]  x value for the X-axis component of @p vect
 */
#define rtssVect3SetX(vect, x) ((vect).data[0] = (x))


/**
 * @brief Set the Y-axis component of an vect3 object
 * @param[out] vect modified vect3 object
 * @param[in]  y value for the Y-axis component of @p vect
 */
#define rtssVect3SetY(vect, y) ((vect).data[1] = (y))

/**
 * @brief Set the Z-axis component of an vect3 object
 * @param[out] vect modified vect3 object
 * @param[in]  z value for the Z-axis component of @p vect
 */
#define rtssVect3SetZ(vect, z) ((vect).data[2] = (z))

/**
 * @brief Set the data of an vect3 object by explicitly providing all the components
 * @param[out] vect modified vect3 object
 * @param[in]  x X-axis component
 * @param[in]  y Y-axis component
 * @param[in]  z Z-axis component
 */
#define rtssVect3SetAll(vect, x, y, z) ((vect).data[0] = (x), (vect).data[1] = (y), (vect).data[2] = (z))

/**
 * @brief Set all the components of an vect3 object using a 3-elements array of double
 * @param[out] vect modified vect3 object
 * @param[in]  ar 3-elements array of double
 */
#define rtssVect3Set(vect, ar) ((vect).data[0] = (ar)[0], (vect).data[1] = (ar)[1], (vect).data[2] = (ar)[2])

/**
 * @brief Get the address of first data of an vect3 object
 * @param[in] vect vect3 object
 * @return pointer to first element of the vector data
 */
#define rtssVect3Num(vect) ((vect).data)

/**
 * @brief Explicit extraction of a component of an vect3 object
 * @param[in] vect vect3 object
 * @param[in] i index of i-th row (from 1 to 3) of the translational component
 * @return the element i-th in the vector data
 */
#define rtssVect3NumElem(vect, i) ((vect).data[i-1])

/**
 * @brief Explicit extraction of data contained in an vect3 object
 * @param[out] x X-axis component of vector @p vect
 * @param[out] y Y-axis component of vector @p vect
 * @param[out] z Z-axis component of vector @p vect
 * @param[in]  vect vect3 object
 */
#define rtssVect3NumAll(x, y, z, vect) ((x) = (vect).data[0], (y) = (vect).data[1], (z) = (vect).data[2])


/**
 * @brief Data structure describing a square matrix with 3 rows (and columns)
 *
 * Data are stored in Column Major Order.
 */
typedef struct math_matrix33_ {

    double data[9]; /**< matrix data in Column Major Order */

} matrix33;

/**
 * @brief Set the data of an matrix33 object using a 9-elements array of double
 * @param[out] matrix modified matrix33 object
 * @param[in]  bdar 9-elements array of double
 */
#define rtssMatrix33Set(matrix, bdar) \
(\
    (matrix).data[0] = (bdar)[0], (matrix).data[3] = (bdar)[3], (matrix).data[6] = (bdar)[6], \
    (matrix).data[1] = (bdar)[1], (matrix).data[4] = (bdar)[4], (matrix).data[7] = (bdar)[7], \
    (matrix).data[2] = (bdar)[2], (matrix).data[5] = (bdar)[5], (matrix).data[8] = (bdar)[8] \
)

/**
 * @brief Set the data of a symmetric 3-rows square matrix using a 6-elements array of double
 * @param[out] M modified symmetric 3-rows square matrix
 * @param[in]  v array of double whose six elements are \f$M_{11}\f$,
 *              \f$M_{22}\f$, \f$M_{33}\f$, \f$M_{12}\f$, \f$M_{23}\f$ and
 *              \f$M_{13}\f$
 */
#define rtssMatrix33SymSet(M, v) \
( \
    (M).data[0] = (v)[0], (M).data[3] = (v)[3], (M).data[6] = (v)[5], \
    (M).data[1] = (v)[3], (M).data[4] = (v)[1], (M).data[7] = (v)[4], \
    (M).data[2] = (v)[5], (M).data[5] = (v)[4], (M).data[8] = (v)[2] \
)

/**
 * @brief Set the data of an matrix33 object by explicitly providing all the components
 * @param[out] M matrix33 object
 * @param[in]  M11, M12, M13 first row's components of the matrix @p M
 * @param[in]  M21, M22, M23 second row's components of the matrix @p M
 * @param[in]  M31, M32, M33 third row's components of the matrix @p M
 */
#define rtssMatrix33SetAll(M, M11, M12, M13, M21, M22, M23, M31, M32, M33) \
( \
    (M).data[0] = (M11), (M).data[3] = (M12), (M).data[6] = (M13), \
    (M).data[1] = (M21), (M).data[4] = (M22), (M).data[7] = (M23), \
    (M).data[2] = (M31), (M).data[5] = (M32), (M).data[8] = (M33) \
)

/**
 * @brief Get the address of first data codified in an matrix33 object
 * @param[in] matrix matrix33 object
 * @return pointer to first element of the matrix data
 */
#define rtssMatrix33Num(matrix) ((matrix).data)

/**
 * @brief Explicit extraction of an element in an matrix33 object
 * @param[in] matrix matrix33 object
 * @param[in]  i index of i-th row (from 1 to 3)
 * @param[in]  j index of j-th column (from 1 to 3)
 * @return the element (i,j) in the matrix
 */
#define rtssMatrix33NumElem(matrix, i, j) ((matrix).data[(i-1)+(j-1)*3])

/**
 * @brief Explicit extraction of data contained in an matrix33 object
 * @param[out] M11, M12, M13 first row's components of the matrix @p M
 * @param[out] M21, M22, M23 second row's components of the matrix @p M
 * @param[out] M31, M32, M33 third row's components of the matrix @p M
 * @param[in]  M matrix33 object
 */
#define rtssMatrix33NumAll(M11, M12, M13, M21, M22, M23, M31, M32, M33, M) \
( \
    (M11) = (M).data[0], (M12) = (M).data[3], (M13) = (M).data[6], \
    (M21) = (M).data[1], (M22) = (M).data[4], (M23) = (M).data[7], \
    (M31) = (M).data[2], (M32) = (M).data[5], (M33) = (M).data[8] \
)

/**
 * @brief Data structure to be used for inverting a square matrix of order <code>n</code>
 */
typedef struct math_imatrix_ {

    int * ipiv;     /**< integer array of pivot indices */
    double * dwork; /**< double precision (workspace) array */

} imatrix;

/**
 * @brief Allocate a workspace in a Scicos block, for inverse matrix computation
 */
#define rtssWSpaceIMatrixInit(wimptr,dof) if(imatrix_init(wimptr,dof) == FAILURE) {err_call();}

/**
 * @brief De-allocate a workspace, used for inverse matrix computation
 *
 * @param[out] wimptr freed pointer to imatrix objects
 */
#define rtssWSpaceIMatrixDestroy(wimptr) if(imatrix_destroy(wimptr) == FAILURE) {err_call();}

/**
 * @brief Service routine for the allocation of a workspace, for inverse matrix computation
 */
int imatrix_init(imatrix * , int);

/**
 * @brief Service routine for destructing a workspace used for inverse matrix computation
 */
int imatrix_destroy(imatrix *);

/**
 * @brief Data structure to be used for performing an n-dimensional linear interpolation
 *        between real vectors
 *
 * @note  Inspired to the routine for n-dimensional linear interpolation
 * written by Bruno Pincon (Bruno.Pincon@iecn.u-nancy.fr) for Scilab-4.x
 *
 * @note References: SCI/routines/calelm/someinterp.c
 */
typedef struct math_linterpn_ {

    int * dimt;     /**< number of points in the generic dimension */
    int n;          /**< number of dimensions */
    double * u;     /**< workspace array */
    double * v;     /**< workspace array */
    int * k;        /**< workspace array */
    int * ad;       /**< workspace array */

} linterpn;

/**
 * @brief Allocate a workspace in a Scicos block, for n-dimensional linear interpolation
 *
 * @param[in]  dimt  number of points in the generic dimension
 * @param[in]  n     number of dimensions
 * @param[out] wlint pointer to the workspace for n-dimensional linear interpolation
 */
#define rtssWSpaceLInterpNInit(wlint,dimt,n) if(linterpn_init(wlint,dimt,n) == FAILURE) {err_call();}

/**
 * @brief De-allocate a workspace, used for n-dimensional linear interpolation
 *
 * @param[out] wlint freed pointer to linterpn objects
 */
#define rtssWSpaceLInterpNDestroy(wlint) if(linterpn_destroy(wlint) == FAILURE) {err_call();}

/**
 * @brief Service routine for the allocation of a workspace, for n-dimensional linear interpolation
 */
int linterpn_init(linterpn * , int , int);

/**
 * @brief Service routine for destructing a workspace used for n-dimensional linear interpolation
 */
int linterpn_destroy(linterpn *);

/**
 * @brief Constant times a vector plus a vector
 */
int math_daxpy_(int , double , double * , double *);

/**
 * @brief Forms the dot product of two vectors
 */
double math_ddot(int , double * , double *);

/**
 * @brief 3D vector cross product
 */
int math_cross_(double * , double , double * , double , double *);

/**
 * @brief Generic Matrix vector multiply
 */
int math_dgemv_(double , enum math_enum_ , int , int , double * , double * , double , double *);

/**
 * @brief Symmetric Matrix vector multiply
 */
int math_dsymv_(double , int , double * , double * , double , double *);

/**
 * @brief Generic Matrix matrix multiply
 */
int math_dgemm_(double , enum math_enum_ , int , int , double * , enum math_enum_ , int , double * , double , double *);

/**
 * @brief Generic matrix inversion
 */
int math_imat_(int , double * , int * , double *);

/**
 * @brief one-dimensional linear interpolation of real vectors
 */
int math_linterp1_(double * , double * , int , double * , double * , linterpn *);

/**
 * @brief Error checking for function math_daxpy_()
 */
#define math_daxpy(N, alpha, x, y) if(math_daxpy_(N, alpha, x, y) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_cross_()
 */
#define math_cross(c, alpha, a, beta, b) if(math_cross_(c, alpha, a, beta, b) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_dgemv_()
 */
#define math_dgemv(alpha, trans, mA, nA, A, x, beta, y) if(math_dgemv_(alpha, trans, mA, nA, A, x, beta, y) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_dsymv_()
 */
#define math_dsymv(alpha, nA, A, x, beta, y) if(math_dsymv_(alpha, nA, A, x, beta, y) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_dgemm_()
 */
#define math_dgemm(alpha, transA, m, k, A, transB, n, B, beta, C) if(math_dgemm_(alpha, transA, m, k, A, transB, n, B, beta, C) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_imat_()
 */
#define math_imat(mA, A, ipiv, dwork) if(math_imat_(mA, A, ipiv, dwork) == FAILURE) {err_call();}

/**
 * @brief Error checking for function math_linterp1_()
 */
#define math_linterp1(yp, t, my, y, t_star, wlint) if(math_linterp1_(yp, t, my, y, t_star, wlint) == FAILURE) {err_call();}

/** @} */
#endif /* MATH_HDR */
