/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include <memory.h>
#include "scicos_block4.h"
#include "error.h"     /* Interface to ERROR module */
#include "robot.h"     /* Interface to ROBOT submodule */


/**
 * @brief Service routine which initializes an robot object from a Scicos block
 *
 * A more detailed description goes here (TODO).
 *
 * @param[out] rp  pointer to the initialized object
 * @param[in]  blk pointer to the structure containing Scicos block's information
 * @return     SUCCESS if all has gone well, FAILURE otherwise
 * @note       TODO: setting link's qlim from qlim_matrix is still missing.
 */
int
robot_init(robot * rp, scicos_block * block)
{
    SCSREAL_COP * legacy_matrix, * mdh, * offset_vector, * qlim_matrix, * gravity_vector, * base_frame, * tool_frame;
    int i, j, N, m;

    if (!rp)
        err_null_ptr();

    /* rebuild the robot object from the "unwrapped" robot stored in model.opar */
    /*                                                                          */
    /* elements from 2 to 4 are descriptive parameters */
    /* They aren't currently  supported.               */
    /*                                                 */
    /* getting elements 5: the constituent links list, */
    /* element 6: number of constituent links          */
    /* and element 7: mdh.                             */
    legacy_matrix = GetRealOparPtrs(block, 1);
    N = GetOparSize(block, 1, 1);
    m = GetOparSize(block, 1, 2);
    mdh = GetRealOparPtrs(block, 2);
    offset_vector = GetRealOparPtrs(block, 3);
    qlim_matrix = GetRealOparPtrs(block, 4);
    rp->dof = N;    /* preparing robot structure for the links */

    rp->links = scicos_malloc(N * sizeof(link)) /* allocate memory for links */
    if (rp->links == NULL){
        set_block_error(-16);
        return;
    }    
    
    rp->mdh = (int)(*mdh);
    switch (m) {

        case LEGACY_DYN: {
            /* create a link from a legacy DYN matrix */
            SCSREAL_COP legacy_dyn_row[LEGACY_DYN]; /* needed to initialize link COG vector and link */
                                                         /* inertia matrix by using the corresponding macros */
            for(i = 0; i < N; i++) {
                for(j = 0; j < LEGACY_DYN; j++)
                    legacy_dyn_row[j] = legacy_matrix[i + N*j];
                rtssLinkInit(&rp->links[i]); /* initialize the j-th link structure */
                rp->links[i].alpha = legacy_dyn_row[0];
                rp->links[i].A = legacy_dyn_row[1];
                rp->links[i].theta = legacy_dyn_row[2];
                rp->links[i].D = legacy_dyn_row[3];
                rp->links[i].sigma = (int) legacy_dyn_row[4];
                rp->links[i].mdh = (int)(*mdh);
                rp->links[i].offset = offset_vector[i];
                rp->links[i].m = legacy_dyn_row[5];
                rtssLinkSetCOG(rp->links[i], &legacy_dyn_row[6]);
                rtssLinkSetInertia(rp->links[i], &legacy_dyn_row[9], 0); /* 0 is FALSE_ */
                rp->links[i].Jm = legacy_dyn_row[15];
                rp->links[i].G = legacy_dyn_row[16];
                rp->links[i].B = legacy_dyn_row[17];
                rp->links[i].Tc[0] = legacy_dyn_row[18]; rp->links[i].Tc[1] = legacy_dyn_row[19];
                /* TODO QLIM */
            }
            break;
        }

        case LEGACY_DH:
            /* create a link from a legacy DH matrix */
            for(i = 0; i < N; i++) {
                rtssLinkInit(&rp->links[i]); /* initialize the j-th link structure */
                rp->links[i].alpha = legacy_matrix[i];
                rp->links[i].A = legacy_matrix[i + N];
                rp->links[i].theta = legacy_matrix[i + N*2];
                rp->links[i].D = legacy_matrix[i + N*3];
                rp->links[i].sigma = (int) legacy_matrix[i + N*4];
                rp->links[i].mdh = (int)(*mdh);
                rp->links[i].offset = offset_vector[i];
                /* TODO QLIM */
            }
            break;

    }

    /* getting element 8: gravity */
    gravity_vector = GetRealOparPtrs(block, 5);
    rtssRobotSetGravity(*rp, gravity_vector);

    /* getting element 9: base    */
    base_frame = GetRealOparPtrs(block, 6);
    rtssRobotSetBase(*rp, base_frame);

    /* getting element 10: tool   */
    tool_frame = GetRealOparPtrs(block, 7);
    rtssRobotSetTool(*rp, tool_frame);
    return(SUCCESS);
}
